
from rusty.models import Model
from rusty.views import View
from rusty.utils import fqcn
from rusty.reactor import Connection

class AdminView(View):
    def models(self, request):
        """
        List models
        """
        models = []
        for model in Model.get_descendants():
            d = {
                "abstract": model.abstract,
                "slug": model._get_name(),
                "fqcn": fqcn(model),
                "count": model.objects.count() if hasattr(model, "objects") else None, # Instance inspector
                "fields": dict([(name, fqcn(field)) for name,field in model._fields.items()]) if hasattr(model, "_fields") else None, # Schema inspector
            }
            models.append(d)
        return models
        
    def workers(self, request):
        return [worker._get_name() for worker in Worker.get_descendants()]
        
    def connections(self, request):
        return Connection.objects.all()
                
    def instances(self, request, fqcn):
        """
        List objects by fully qualified class name
        """
        pass
    
    def retrieve(self, request, fqcn, pk):
        """
        Retrieve object by fully qualified class name and primary key
        """
        pass
        

    def schema(self, request, fqcn):
        """
        Retrieve class schema by fully qualified class name
        """
        pass
