
from rusty.models import Model, Manager
from rusty.controllers import Controller
from rusty.fields import SlugField, StringField, IntegerField
from rusty.templating import JinjaEnvironmentField

class ApplicationManager(Manager):
    PRIMARY_KEY = "slug"

class Application(Model):
    objects = ApplicationManager()

    slug = SlugField(
        description="Application slug")
    name = StringField(
        description="Application name")
    facebook_id = IntegerField(
        description="Facebook id of the application")
    lastfm_api_key = StringField(
        description="Last.fm API key")
    lastfm_api_secret = StringField(
        description="Last.fm API secret")
    jinja_environment=JinjaEnvironmentField(
        description="Jinja environment or template root")

class ApplicationController(Controller):
    def __init__(self, application, *controllers):
        super(ApplicationController, self).__init__(*controllers)
        self.application = application if isinstance(application, Application) else Application.objects.get(slug=application)
        
    def pre(self, request):
        request.application = self.application

Application.prepare()
