
import os
import re
import codecs
from rusty.models import Model, Manager
from rusty.fields import StringField, DateTimeField, DateField, SlugField, BooleanField
from rusty.views import View
from rusty.http import FoundResponse
from rusty.templating import JinjaResponse

class BlogPostManager(Manager):
    def load(self, root):
        def parse_headers(chunk):
            headers = {}
            for line in chunk.split("\n"):
                m = re.match("\.\.\s+\-x\-(\w+)\:\:?\s*(.+)?", line)
                assert m, "Failed to parse headers from: '%s'" % line
                key, value = m.groups()
                headers[key] = value
            return headers
            
        for filename in sorted(os.listdir(root), reverse=True):
            m = re.match("(?P<published>\d{4}-\d\d-\d\d)\-(?P<slug>[\w\d\-]+)\.rst$", filename.lower())
            if not m:
                continue
            fh = codecs.open(os.path.join(root, filename), encoding="utf-8")
            headers, content = fh.read().split("\n\n", 1)
            properties = m.groupdict()
            properties.update(parse_headers(headers))
            properties["content"] = content
            self.create(**properties)
        

class BlogPost(Model):
    objects = BlogPostManager()
    
    slug = SlugField()
    author = StringField()
    published = DateField()
    hidden = BooleanField()
    title = StringField()
    content = StringField()
    
class BlogView(View):
    def __init__(self, template=None):
        self.template = template

    def list(self, request, **args):
        return BlogPost.objects.all()

    def dispatch(self, request):
        if request.headers.get("X-Requested-With", None) == "XMLHttpRequest":
            return super(BlogView, self).dispatch(request)

        if request.path:
            if request.path.endswith("/"):
                request.path = request.path[:-1]

            try:
                posts = BlogPost.objects.get(slug=request.path),
            except BlogPost.DoesNotExist:
                return FoundResponse("/blog/")
        else:
            posts = BlogPost.objects.filter(lambda b:not b.hidden).all()

        return JinjaResponse(
            template=self.template or "blog.html",
            context=dict(
                posts=posts[:10]
            ))

        
BlogPost.prepare()

