

from rusty.controllers import Controller

import struct
import tarfile
import time
import types
import zlib
from rusty.pipeline import Element

class Tar(Element):
    def pipe(self, haystack):
        for needle in haystack:
            assert needle.is_file
            tarinfo = tarfile.TarInfo(needle.filename)
            tarinfo.size = needle.size
            tarinfo.mode = 0755
            tarinfo.type = tarfile.REGTYPE
            yield tarinfo.tobuf(tarfile.PAX_FORMAT)
            fh = needle.get_read_handle()
            while True:
                chunk = fh.read(512)
                yield chunk
                if len(chunk) < 512:
                    yield "\0" * (512-len(chunk))
                    break

class Gzip(Element):
    def __init__(self, compression_level=9):
        self.compression_level = compression_level
        
    def pipe(self, haystack):
        """
        Shameless copypaste from CherryPy gzipfilter
        """
        
        # Write header
        yield '\037\213\010\0'
        yield struct.pack("<L", long(time.time()))
        yield '\002\377'
        
        crc = zlib.crc32("")
        size = 0
        zobj = zlib.compressobj(self.compression_level,
                                zlib.DEFLATED, -zlib.MAX_WBITS,
                                zlib.DEF_MEM_LEVEL, 0)
                                
        # If the haystack is a generator, assume it produces only strings
        if hasattr(haystack, "__iter__"):
            for needle in haystack:
                assert isinstance(needle, basestring), "Invalid needle in haystack: %s" % needle
                crc = zlib.crc32(needle, crc)
                size += len(needle)
                yield zobj.compress(needle)

        # If it is a file, read the content
        elif isinstance(haystack, types.FileType):
            # TODO: Perhaps use external process to reduce kernelspace-userspace copies,
            #       that could cause additional latencies though
            while True:
                chunk = haystack.read(65536)
                if not chunk:
                    break
                crc = zlib.crc32(chunk, crc)
                size += len(chunk)
                yield zobj.compress(chunk)

        # Otherwise just write the response as string
        elif isinstance(haystack, basestring):
            crc = zlib.crc32(haystack, crc)
            size += len(haystack)
            yield zobj.compress(haystack)
        else:
            raise ValueError("Do not know how to handle haystack of type %s" % type(haystack))   

        yield zobj.flush()
        yield struct.pack("<l", crc)
        yield struct.pack("<L", size & 0xFFFFFFFFL)

