
import types
from rusty.jason import JasonEncoder
from rusty.http import Response

def serialize(mapper=lambda obj:obj.serialize() if hasattr(obj, "serialize") else obj, mimetype="application/json"):
    """
    Serialize objects array returned by the target function
    """
    def decorator(target_func):
        def decorated(request, *args, **kwargs):
            mixed = target_func(request, *args, **kwargs)
            encoder = ExtendedEncoder(indent=4)
            try:
                if isinstance(mixed, types.DictType):
                    response = Response(encoder.encode(mapper(mixed)))
                elif hasattr(mixed, "__iter__"):
                    def generator():
                        yield "["
                        first = True
                        for obj in mixed:
                            if not first:
                                yield ", "
                            first = False
                            yield encoder.encode(mapper(obj))
                        yield "]"
                    response = Response(generator())
                else:
                    response = Response(encoder.encode(mapper(mixed)))
                response.headers["Content-Type"] = mimetype
                return response
            except TypeError, e:
                raise TypeError("Unable to serialize:", mixed)
        return decorated
    return decorator
