
from rusty.signals import Signal
from rusty.controllers import Controller
from rusty.fields import DateTimeField, UUIDField, ForeignKey
from rusty.application import Application
from rusty.authentication import User
from rusty.models import Model
from rusty.terminal import Colors

class EventManager(object):
    PRIMARY_KEY = "uuid"
    
    def __init__(self):
        self.created = Signal()

    def count(self):
        return 0
        
    def set_model(self, model):
        self.model = model
        
    def create(self, **properties):
        event_name = properties.pop("name")
        for cls in Event.get_descendants():
            if event_name == cls._get_name():
                return cls(**properties)
        raise Event.DoesNotExist

class Event(Model):
    objects = EventManager()
    origin = UUIDField()
    application = ForeignKey(
        Application,
        description="Related application")
    user = ForeignKey(
        User,
        description="Related user")

    def __init__(self, **properties):
        super(Event, self).__init__(**properties)
        if self.__class__.__name__ != "Event":
            self._bind_fields()
        self.objects.created.emit(self)
        self.name = self._get_name()
        self.handle()
        print Colors.GRAY + "--- Emitted", self.__class__.__name__, "by", self.user, Colors.END
        
    def handle(self):
        pass
        
class DeviceConnectedEvent(Event):
    """
    Device registers for events via WebSocket/EventSource
    """
    pass
    
class DeviceDisconnectedEvent(Event):
    """
    Device hangs up events WebSocket/EventSource
    """
    pass
    
class EventController(Controller):
    def dispatch(self, request):
        if request.headers.get("Accept", None) == "text/event-stream":
            # Hook up EventSource
            pass
        elif request.headers.get("Upgrade") == "websocket":
            # Hook up WebSocket
            pass
        else:
            # Use GET/POST for emitting events
            pass
    
