
from rusty.location import Country
from rusty.models import Model, Manager
from rusty.views import View
from rusty.fields import StringField, IntegerField, ForeignKey

class HardwareVendorManager(Manager):
    PRIMARY_KEY = "oui"
    
    def load(self, filename):
        fh = open(filename)
        headers, companies = fh.read().strip().split("\n  \n  \n  ")
        
        country_mapping = {}
        for country in Country.objects.all():
            key = country.name.lower()[:34]
            country_mapping[key] = country
            
        for company in companies.split("\n\n"):
            lines = company.split("\n")
            
            if "PRIVATE" in lines[0]:
                continue
            
            assert "(hex)" in lines[0], "(hex) not contained in '%s'" % lines
            assert "(base 16)" in lines[1]
        
            oui, name = lines[1].strip().split("     (base 16)\t\t")
            oui = int(oui, 16)
        
            country_slug = lines[-1].strip().lower()[:34]
        
            address = ", ".join([line.strip() for line in lines[2:-1]])
            
            try:
                country = country_mapping[country_slug]
            except KeyError:
                print "Didnt find country '%s' for company  %s (%s)" %(country_slug, name, oui)
                country = None
                
            self.create(
                    oui=oui,
                    name=name,
                    country = country,
                    address=address)
        print "Loaded %d hardware vendors" % self.count()

class HardwareVendor(Model):
    """
    List of hardware vendors from
    http://standards.ieee.org/develop/regauth/oui/oui.txt
    """
    uuid = None
    created = None
    objects = HardwareVendorManager()

    oui = IntegerField() #OUIField()
    name = StringField() # Group companies because multiple OUIs refer to same company
    address = StringField()
    country = ForeignKey(Country)
    
    def __repr__(self):
        return "%06x" % self.oui
    
class HardwareVendorView(View):
    def list(self, request):
        return HardwareVendor.objects.all()
        
HardwareVendor.prepare()
