# -*- coding: utf-8 -*-

"""
Portable JSON codec wrapper
"""

import warnings
import types
from uuid import UUID
from datetime import datetime, date, time
from json import JSONEncoder

try:
    from cjson import decode as json_decode, encode as json_encode, DecodeError as JasonDecodeError

    SLASHED_STRING = "\"some/string\""

    assert json_decode(json_encode(SLASHED_STRING)) == SLASHED_STRING, \
        "You're running old buggy version of cjson, please upgrade"
except ImportError:
    from json import loads as json_decode, dumps as json_encode
    JasonDecodeError = ValueError
    warning.warn("Falling back to Python implementation for JSON, please install cjson package")

class JasonEncoder(JSONEncoder):
    """
    Handle datetime, date and time objects
    """
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        elif isinstance(obj, time):
            return obj.strftime('%H:%M:%S')                
        elif isinstance(obj, str):
            return unicode(obj)
        elif isinstance(obj, UUID):
            return str(obj)
        else:
            return JSONEncoder.default(self, obj)
