
# -*- coding: utf-8 -*-

import binascii
from base64 import b64decode
import email
import email.header
import imaplib
from rusty.models import Model, Manager
from rusty.views import View
from rusty.authentication import  User
from rusty.fields import StringField, PortField, ForeignKey, BooleanField, \
    MD5Field, ListField, MimetypeField, EmailField
from threading import Thread
from rusty.http import Response, NotFoundResponse
from rusty.events import Event

class ExternalInboxWorker(Thread):
    def __init__(self):
        super(MonitoringThread, self).__init__()
        self.daemon = True

class ExternalInbox(Model):
    PRIMARY_KEY = "email"
    
    """
    External inbox represents IMAP inbox
    """
    email = EmailField()
    owner = ForeignKey(User)
    merge = BooleanField(
        default=True)
    protocol = StringField() # eg. imap, imap+
    hostname = StringField()
    port = PortField()
    encryption = StringField() # ssl, tls
    username = StringField()
    password = StringField()

    def pull(self):
        connection = imaplib.IMAP4_SSL(self.hostname)
        connection.login(self.username, self.password)
        connection.select("inbox")
        result, data = connection.uid("search", None, "ALL")
        uids = data[0].split()

        for uid in uids[:-50:-1]:
            status, result = connection.uid('fetch', uid, '(RFC822)')
            result, _ = result # Skip this weird ")"
            _, result = result # Skip UID replica


            m = email.message_from_string(result)
            message_id = m.get("Message-ID")
            assert message_id.startswith("<") and message_id.endswith(">") and "/" not in message_id, \
                "Invalid Message-ID: %s" % message_id
            message_id = message_id[1:-1]

            def parse_bits(mixed):
                return "".join([bit.decode(encoding) if encoding else bit \
                    for bit, encoding in email.header.decode_header(mixed)])


            message_thread_index = m.get("Thread-Index")

            if message_thread_index:
                message_thread_index = b64decode(message_thread_index)
                conversation = Conversation.objects.get_or_create(
                    uuid = binascii.hexlify(message_thread_index[:22]))
            else:
                conversation = None

            (sender_full_name, sender_email), = email.utils.getaddresses(m.get_all("From"))
            sender_full_name = parse_bits(sender_full_name)
            sender_email = sender_email.lower()

            if " " in sender_full_name:
                sender_first_name, sender_last_name = sender_full_name.rsplit(" ", 1)
            else:
                sender_first_name, sender_last_name = sender_full_name, None
            sender = User.objects.get_or_create(
                email = sender_email)

            sender.update(
                first_name = sender_first_name,
                last_name = sender_last_name)

            try:
                Message.objects.get(
                    uuid=message_id)
            except Message.DoesNotExist:

                message = Message.objects.create(
                    conversation = conversation,
                    sender = sender,
                    recipient = self.owner,
                    uuid = message_id,
                    subject = parse_bits(m.get("Subject")))

                first = True

                main_part = None

                for part in m.walk():
                    part_content_type, part_charset = part.get_content_type(), part.get_content_charset()                
                    print "Fetching part of:", parse_bits(m.get("Subject")), part_content_type, part_charset
                    content_id = part.get("Content-ID")
                    
                    if content_id:
                        assert content_id.startswith("<") and content_id.endswith(">") and "/" not in content_id, \
                            "Invalid Message-ID: %s" % content_id
                        content_id = content_id[1:-1]

                    filename = part.get_filename()
                    payload = part.get_payload(decode=True)

                    if part_charset and part_charset.startswith("text/"):
                        payload = unicode(payload, part_charset)

                    if filename or part_content_type in ("text/html", "text/plain"):
                        message_part = MessagePart.objects.create(
                            content_id = content_id,
                            mimetype = part_content_type,
                            message = message,
                            filename = filename,
                            content = payload)

class AttachmentManager(Manager):
    pass

class Attachment(Model):
    objects = AttachmentManager()

    md5 = MD5Field()
    filename = StringField()
    mimetype = StringField()

class ConversationManager(Manager):
    pass

class Conversation(Model):
    objects = ConversationManager()
    uuid = StringField(
        description="In case of an e-mail this is the Thread-Index field")

class MessageManager(Manager):
    pass

class Message(Model):
    PRIMARY_KEY = "uuid"
    
    mailbox = ForeignKey(ExternalInbox)
    conversation = ForeignKey(Conversation)
    uuid = StringField(
        description="In case of an e-mail this is the Message-ID")
    sender = ForeignKey(
        User)
    recipient = ForeignKey(
        User)
    subject = StringField(
        description="Message subject")

class MessagePart(Model):
    PRIMARY_KEY = "uuid"
    INDEXES = ("message", "content_id"),
    
    message = ForeignKey(
        Message)
    mimetype = MimetypeField()
    filename = StringField()
    content_id = StringField(
        description="Content-ID header")
    content = StringField()


class InboxView(View):
    def part(self, request, message_uuid=None, content_id=None):
        try:
            part = MessagePart.objects.get(
                uuid=message_uuid,
                content_id=content_id)
        except MessagePart.DoesNotExist:
            return NotFoundResponse()
        else:
            return Response(
                headers = {
                    "Content-Type": part.mimetype,
                    "Content-Disposition": "attachment" },
                body = part.content)
        
    def raw(self, request, uuid):
        try:
            part = MessagePart.objects.get(
                uuid=uuid)
        except MessagePart.DoesNotExist:
            return NotFoundResponse()
        else:
            return Response(
                headers = {"Content-Type": part.mimetype, "Content-Disposition": "attachment" },
                body = part.content)
            
    def retrieve(self, request, uuid, unwrap=True):
        try:
            parts = MessagePart.objects.filter(
                lambda part: part.message.uuid == uuid and part.mimetype == "text/html").all()
            part, = parts
        except ValueError:
            part, = MessagePart.objects.filter(
                lambda part: part.message.uuid == uuid and part.mimetype == "text/plain")
            part_content = part.content.replace("\n\r", "\n").replace("\r", "")
            
            part_content = "".join([line for line in part_content if not line.startswith(">")])
            
            if "\n-----Original Message-----\n" in part_content:
                part_content, _ = part_content.split("\n-----Original Message-----\n", 1)
            part_encoding = "utf-8"

            part_content = "<pre>" + part_content + "</pre>"
        else:
            from BeautifulSoup import BeautifulSoup, Comment
            soup = BeautifulSoup(part.content)
            
            part_encoding = "utf-8"
            
            # Remove comments
            for tag in soup.findAll(text=lambda text:isinstance(text, Comment)):
                tag.extract()


            # Replace embedded image href-s
            for tag in soup.findAll("img"):
                if tag["src"].startswith("cid:"):
                    tag["src"] = "/inbox/part/?message_uuid=" + part.message.uuid + "&content_id=" + tag["src"][4:]
            
            # Unwrap body content
            if unwrap:

                for tag in soup.findAll():
                    # Strip style and script tags
                    if tag.name.lower() in ("style", "script"):
                        tag.extract()  

                    # Remove selectors
                    for attribute in ["class", "id", "name"]:
                        del tag[attribute]

                if soup.body:
                    soup = soup.body
                    soup.name = "div" # Replace body with div
                    # TODO: Unwrap excessive div-s
                    
            part_content = unicode(soup)

        return Response(
            headers = {"Content-Type": "%s; charset=%s" % (part.mimetype, part_encoding) if part_encoding else part.mimetype },
            body = part_content)

    def list(self, request):
        assert request.user, "User not authenticated"
        return Message.objects.filter(
            lambda m:m.recipient == request.user).all()

class ConversationView(View):
    def list(self, request):
        assert request.user, "User not authenticated"
        return Conversation.objects.all()
        

class MessageReceivedEvent(Event):
    conversation = ForeignKey(Conversation)
    message = ForeignKey(Message)


ExternalInbox.prepare()
Message.prepare()
MessagePart.prepare()
Conversation.prepare()

