
import netifaces
from rusty.fields import IPv4Field, StringField, NetmaskField, ForeignKey
from rusty.hardware_vendors import HardwareVendor
from rusty.models import Manager, Model

class NetworkInterfaceManager(Manager):
    def sync(self):
        for device in netifaces.interfaces():
            addresses = netifaces.ifaddresses(device)
            
            try:
                # OpenVZ virtual containers do not have hardware address!
                hardware, = addresses.pop(netifaces.AF_PACKET)
                hardware_address = hardware.pop("addr")
            except KeyError:
                hardware_address = None
            
            if hardware_address:
                oui = hardware_address[0:2] + hardware_address[3:5] + hardware_address[6:8]
                try:
                    vendor = HardwareVendor.objects.get(oui=int(oui, 16))
                except HardwareVendor.DoesNotExist:
                    print "Could not find hardware vendor matching OUI %s" % oui
                    vendor = None
            vendor = None
            
            try:
                # Not enabled interface does not have IP address assigned to it
                logical_address, = addresses.pop(netifaces.AF_INET)
                address = logical_address.pop("addr")
                netmask = logical_address.pop("netmask")
                broadcast = logical_address.pop("boardcast", None)
            except KeyError:
                address = None
                netmask = None
                broadcast = None
                
            network_interface = self.create(
                device=device,
                hardware=hardware_address,
                vendor=vendor,
                address=address,
                netmask=netmask,
                broadcast=broadcast)

        print "Loaded %d network interfaces" % self.count()

class NetworkInterface(Model):
    objects = NetworkInterfaceManager()
    
    device = StringField()
    address = IPv4Field()
    hardware = StringField() # TODO: HardwareAddressField()
    vendor = ForeignKey(HardwareVendor)
    netmask = NetmaskField()
    broadcast = StringField()
    
    def __repr__(self):
        return "<NetworkInterface %s (%s) at %s>" % (
            self.device,
            self.hardware if self.hardware else "No hardware address",
            self.address)
            
NetworkInterface.prepare()
NetworkInterface.objects.sync()

