
import re
from rusty.controllers import Controller
from rusty.fields import DateTimeField, ForeignKey, DomainField
from rusty.models import Model, Manager
from rusty.terminal import Colors
from rusty.authentication import User

class Session(Model):
    domain = DomainField()
    user = ForeignKey(User, null=True)
    expires = DateTimeField(null=True)
    
    def get_event_sources():
        EventSource.objects.filter(session=self)

    def __unicode__(self):
        return "<Session uuid=%(uuid)s user=%(user)s domain=%(domain)s>" % self

class SessionController(Controller):
    def __init__(self, domain, *controllers):
        super(SessionController, self).__init__(*controllers)
        assert isinstance(domain, unicode), "Explicit unicode domain required"
        self.domain = domain.encode("idna")

    def __unicode__(self):
        return "<SessionController domain=%s path=/>" % self.domain

    def pre(self, request):
        def get_session():
            cookie = request.headers.get("Cookie", "")
            
            if not cookie:
                raise Session.DoesNotExist("Invalid cookie")
            
            # Naive regex for matching session UUID from cookie,
            # perhaps we should use some standardized library?
            matched = re.match("session_uuid=(?P<uuid>[0-9a-f]{32})", cookie)

            if not matched:
                raise Session.DoesNotExist("Invalid session UUID %s" % cookie)
            return Session.objects.get(**matched.groupdict())
        
        try:
            request.session = get_session()
        except Session.DoesNotExist, e:
            request.session = Session.objects.create(
                domain=self.domain)
                
        # Shortcut!
        request.user = request.session.user if request.session else None
        if request.user:
            print Colors.BLUE + "%%%" + Colors.END,"Logged in as:",
            if request.user.first_name:
                print request.user.first_name,
            if request.user.last_name:
                print request.user.last_name,
            print "<%s>" % request.user.email
        
    def post(self, request, response):
        if request.session:
            response.headers["Set-Cookie"] = "session_uuid=%s; Domain:%s; Path=/;" % (request.session.uuid.hex, self.domain)

Session.prepare()
