
import resource
import os
import re
import socket
from rusty.network_interfaces import NetworkInterface
from rusty.models import Model
from rusty.views import View
from rusty.utils import fqcn
from rusty.decorators import serialize
import platform

class StatsView(View):
    def retrieve(self, request):
        instances = {}
        
        for model in Model.get_descendants():
            if not model.abstract:
                instances[fqcn(model)] = model.objects.count()
            
        d = {}
        
        pid = os.getpid()
        buf = open('/proc/%d/status' % pid).read()
        m = re.search("VmSize:\s*(\d+)\s*kB", buf)
        memory_usage, = m.groups()
        
        
        kernel, hostname, version, release, processor, hardware_platform = platform.uname()

        return dict(
            host=dict(
                hostname=hostname,
                kernel=kernel,
                version=version,
                release=release,
                processor=processor,
                hardware_platform=hardware_platform
                
            ),
            network_interfaces=tuple(NetworkInterface.objects.map(lambda nic:dict(
                device=nic.device,
                hardware=nic.hardware,
                address=nic.address,
                netmask=nic.netmask,
                broadcast=nic.broadcast))),
            python_process=dict(
                pid=pid,
                uid=os.getuid(),
                gid=os.getgid(),
                maxrss_mb = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1024,
                memory_usage_mb=int(memory_usage) / 1024
            ),
            instances=instances)
