
import re
from rusty.models import Manager, Model
from rusty.views import View
from rusty.fields import StringField, IntegerField, BooleanField, ForeignKey

class FingerprintManager(Manager):
    PRIMARY_KEY = "fingerprint"
    INDEXES = ("fingerprint",),

class DeviceType(Model):
    objects = FingerprintManager()

    fingerprint = StringField()   # eg. GT-I9300
    vendor = StringField()        # eg. Samsung, Apple, Nokia
    product = StringField()       # eg. Galaxy S3
    category = StringField()      # eg. phone, tablet, laptop
    
    # android && !mobile => tablet
    # mobile ==>  phone

class OperatingSystem(Model):
    objects = FingerprintManager()
    
    fingerprint = StringField()
    family = StringField()          # windows, ios, linux, android
    flavor = StringField()          # Ubuntu, Cyanogen, Droid
    codename = StringField()        # Jelly Bean, Queezy Quantal
    major = IntegerField()
    minor = IntegerField()
    architecture = StringField()

class Browser(Model):
    objects = FingerprintManager()
    
    fingerprint = StringField()
    bot = BooleanField()            # Whether this is web crawler
    family = StringField()          # Firefox, Internet Explorer
    minor = IntegerField()
    major = IntegerField()


class BrowserEngine(Model):
    objects = FingerprintManager()
    
    fingerprint = StringField()     # Webkit/512.34
    family = StringField()          # Webkit, Trident, Gecko
    minor = IntegerField()
    major = IntegerField()

class UserAgent(Model):
    objects = FingerprintManager()
    
    fingerprint = StringField()
    hits = IntegerField(
        default=0,
        null=False)
    operating_system = ForeignKey(OperatingSystem)
    browser = ForeignKey(Browser)
    engine = ForeignKey(BrowserEngine)
    device = ForeignKey(DeviceType)
    
    banned = BooleanField()         # Whether this user agent is malicious
    confirmed = BooleanField()      # Whether fields of this user agent are confirmed
    
    architecture = StringField()    # x86_64?
#    fragments = ListField(StringField, transient=True)
        
    def __init__(self, **properties):
        super(UserAgent, self).__init__(**properties)
        self.fragments = set()
        for fragment, _, extensions in re.findall("\s*(.+?)(\(\s*(.+?)\s*\))?\s+", self.fingerprint):
            self.fragments.add(fragment)
            self.fragments.update(re.split("\s*;\s*", extensions) if extensions else [])
        
    def hit(self):
        self.hits += 1
        self.save()
        
    def __repr__(self):
        return " ".join([bit for bit in (self.browser, self.engine, self.operating_system, self.device) if bit])
        
class UserAgentView(View):
    def list(self, request):
        return UserAgent.objects.all()
        
#    browser = ForeignKey(Browser)
#    operating_system = ForeignKey(OperatingSystem)
#    engine = ForeignKey(BrowserEngine)
#    device = ForeignKey(Device)
    
"""
    

"""
"""
Phone UA:

Mozilla/5.0 (Linux; <Android Version>; <Build Tag etc.>)
AppleWebKit/<WebKit Rev> (KHTML, like Gecko) Chrome/<Chrome Rev> Mobile
Safari/<WebKit Rev>

Tablet UA:

Mozilla/5.0 (Linux; <Android Version>; <Build Tag etc.>)
AppleWebKit/<WebKit Rev>(KHTML, like Gecko) Chrome/<Chrome Rev>
Safari/<WebKit Rev>
"""

UserAgent.prepare()
Browser.prepare()
BrowserEngine.prepare()
DeviceType.prepare()
OperatingSystem.prepare()
