
import types
from rusty.controllers import Controller
from rusty.models import Model
from rusty.http import FoundResponse, Response
from rusty.jason import JasonEncoder, json_encode
from rusty.decorators import serialize

class View(object):
    def __repr__(self):
        return "<%s>" % (self.__class__.__name__)
        
    def dispatch(self, request):
        methods = dict(self._get_methods())
        
        method = request.path
        if method:
            if method.endswith("/"):
                method = method[:-1]
            assert request.method == "GET"
        else:
            method = request.method.lower()
            if method == "get":
                if "retrieve" in methods:
                    method = "retrieve"
                elif "list" in methods:
                    method = "list"
            if method == "post":
                if "create" in methods:
                    method = "create"

        if method in methods:
            func = methods[method]
            mixed = func(self, request, **request.query)
            encoder = JasonEncoder(indent=4)
            
            if isinstance(mixed, Response):
                return mixed
            
            # Okay, let's serialize then
            if isinstance(mixed, Model):
                response = Response(
                    encoder.encode(mixed.serialize()),
                    headers={"X-Rusty-Model":"%s.%s" % (mixed.__module__, mixed.__class__.__name__)})
            elif isinstance(mixed, types.DictType):
                response = Response(encoder.encode(mixed))
            elif hasattr(mixed, "__iter__"):
                def generator():
                    yield "["
                    first = True
                    for needle in mixed:
                        if not first:
                            yield ", "
                        first = False

                        if isinstance(needle, Model):
                            needle = needle.serialize(namespace=True)
                        try:
                            yield json_encode(needle) # encoder.encode(needle)
                        except Exception, e:
                            print "Failed to encode to JSON:", needle
                            raise e
                    yield "]\n"
                response = Response(generator())
            else:
                raise Exception("Don't know how to interpret %s (%s) as view response" % (mixed, type(mixed)))
            response.headers["Content-Type"] = "application/json; charset=UTF-8"
            response.headers["Access-Control-Allow-Origin"] = "*"
            return response
        else:
            raise Exception("Invalid method: %s" % method)
            
        
    @classmethod
    def _get_methods(cls):
        for attr in dir(cls):
            mixed = getattr(cls, attr)
            if attr.startswith("_"):
                continue
            if attr in ("get",):
                continue
            if not isinstance(mixed, types.MethodType):        
                continue
            yield attr, mixed
    
    @classmethod
    def _get_class_schema(cls):
        d={
            "class":"%s.%s" % (cls.__module__, cls.__name__),
            "methods":{}}
        for name, func in cls._get_methods():
            if hasattr(func, "form"):
                d["methods"][name] = func.form.schema()
            else:
                d["methods"][name] = {}
        return d

    @serialize(mimetype="application/schema+json")
    def schema(self, request):
        return self._get_class_schema()

class RedirectView(View):
    def __init__(self, destination):
        self.destination = destination
        
    def dispatch(self, request):
        return FoundResponse(self.destination)

