
import os
import re
from uuid import UUID
from rusty.fields import IntegerField, PathField
from rusty.models import Model, Manager

"""
Bloatware for wrapping volumes/mountpoints
"""

class VolumeManager(Manager):
    def __init__(self):
        super(VolumeManager, self).__init__()
        
    def scan(self):
        RE_UUID = '[0-9a-f]{8}\-[0-9a-f]{4}\-[0-9a-f]{4}\-[0-9a-f]{4}\-[0-9a-f]{12}'
        PROC_MOUNTS = '/etc/mtab'
        DEV_DISK_BY_UUID = '/dev/disk/by-uuid'
        
        if os.path.exists(DEV_DISK_BY_UUID):
            path_to_uuid_mapping = {}
            for uuid in os.listdir(DEV_DISK_BY_UUID):
                if re.match(RE_UUID, uuid):
                    path_to_uuid_mapping[os.path.realpath(os.path.join(DEV_DISK_BY_UUID, uuid))] = uuid
        else:
            path_to_uuid_mapping = None
        
        path_to_mountpoint_mapping = {}        
        for line in open(PROC_MOUNTS):
            m = re.match('(.+)\s+(.+)\s+(.+)\s+(.+)\s+(\d+)\s+(\d+)\s*', line)
            path, mountpoint, filesystem, options, _, _ = m.groups()
            if not path.startswith("/"):
                continue

            if not os.path.exists(path):
                print "Warning: Volume path does not exist %s" % path

            assert os.path.ismount(mountpoint)
            path_to_mountpoint_mapping[path] = mountpoint

        for path, mountpoint in path_to_mountpoint_mapping.items():
            device_id = os.stat(mountpoint).st_dev
            volume = Volume(
                path=path,
                mountpoint=mountpoint,
                uuid=path_to_uuid_mapping[path] if path_to_uuid_mapping else UUID(int=device_id),
                device=device_id)
            print "Detected volume:", volume
            self.append(volume)

    def get_by_device(self, device_id):
        for volume in self.objects:
            if volume.device == device_id:
                return volume
        raise Volume.DoesNotExist("Volume with device id %d not found" % device_id)

    def get_by_uuid(self, volume_uuid):
        for volume in self.volumes:
            if volume.uuid == volume_uuid:
                return volume
        raise Volume.DoesNotExist("Volume with UUID %s not found" % volume_uuid)

class Volume(Model):
    objects = VolumeManager()
    
    device = IntegerField(
        null=False)
    mountpoint = PathField(
        null=False)
    path = PathField(
        null=False)
        
    def get_absolute_path(self, relative_path):
        if relative_path.startswith("/"):
            relative_path = relative_path[1:]
        absolute_path = os.path.realpath(os.path.join(self.mountpoint, relative_path))
        assert absolute_path.startswith(self.mountpoint), "Invalid path"
        return absolute_path
        
    def __repr__(self):
        return "<Volume %s on device %d with UUID %s mounted at %s>" % (
            self.path,
            self.device,
            self.uuid,
            self.mountpoint)

Volume.objects.scan()
